+{
   locale_version => 1.31,
   entry => <<'ENTRY', # for DUCET v13.0.0
010D      ; [.1FD7.0020.0002] # LATIN SMALL LETTER C WITH CARON
0063 030C ; [.1FD7.0020.0002] # LATIN SMALL LETTER C WITH CARON
010C      ; [.1FD7.0020.0008] # LATIN CAPITAL LETTER C WITH CARON
0043 030C ; [.1FD7.0020.0008] # LATIN CAPITAL LETTER C WITH CARON
0107      ; [.1FD8.0020.0002] # LATIN SMALL LETTER C WITH ACUTE
0063 0301 ; [.1FD8.0020.0002] # LATIN SMALL LETTER C WITH ACUTE
0063 0341 ; [.1FD8.0020.0002] # LATIN SMALL LETTER C WITH ACUTE
0106      ; [.1FD8.0020.0008] # LATIN CAPITAL LETTER C WITH ACUTE
0043 0301 ; [.1FD8.0020.0008] # LATIN CAPITAL LETTER C WITH ACUTE
0043 0341 ; [.1FD8.0020.0008] # LATIN CAPITAL LETTER C WITH ACUTE
011B      ; [.2008.0020.0002] # LATIN SMALL LETTER E WITH CARON
0065 030C ; [.2008.0020.0002] # LATIN SMALL LETTER E WITH CARON
011A      ; [.2008.0020.0008] # LATIN CAPITAL LETTER E WITH CARON
0045 030C ; [.2008.0020.0008] # LATIN CAPITAL LETTER E WITH CARON
0063 0068 ; [.2076.0020.0002] # <LATIN SMALL LETTER C, LATIN SMALL LETTER H>
0063 0048 ; [.2076.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER C, LATIN CAPITAL LETTER H>
0043 0068 ; [.2076.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER C, LATIN SMALL LETTER H>
0043 0048 ; [.2076.0020.0008] # <LATIN CAPITAL LETTER C, LATIN CAPITAL LETTER H>
0142      ; [.20D5.0020.0002] # LATIN SMALL LETTER L WITH STROKE
006C 0335 ; [.20D5.0020.0002] # LATIN SMALL LETTER L WITH STROKE
0141      ; [.20D5.0020.0008] # LATIN CAPITAL LETTER L WITH STROKE
004C 0335 ; [.20D5.0020.0008] # LATIN CAPITAL LETTER L WITH STROKE
0144      ; [.2119.0020.0002] # LATIN SMALL LETTER N WITH ACUTE
006E 0301 ; [.2119.0020.0002] # LATIN SMALL LETTER N WITH ACUTE
006E 0341 ; [.2119.0020.0002] # LATIN SMALL LETTER N WITH ACUTE
0143      ; [.2119.0020.0008] # LATIN CAPITAL LETTER N WITH ACUTE
004E 0301 ; [.2119.0020.0008] # LATIN CAPITAL LETTER N WITH ACUTE
004E 0341 ; [.2119.0020.0008] # LATIN CAPITAL LETTER N WITH ACUTE
0155      ; [.2194.0020.0002] # LATIN SMALL LETTER R WITH ACUTE
0072 0301 ; [.2194.0020.0002] # LATIN SMALL LETTER R WITH ACUTE
0072 0341 ; [.2194.0020.0002] # LATIN SMALL LETTER R WITH ACUTE
0154      ; [.2194.0020.0008] # LATIN CAPITAL LETTER R WITH ACUTE
0052 0301 ; [.2194.0020.0008] # LATIN CAPITAL LETTER R WITH ACUTE
0052 0341 ; [.2194.0020.0008] # LATIN CAPITAL LETTER R WITH ACUTE
0161      ; [.21D3.0020.0002] # LATIN SMALL LETTER S WITH CARON
0073 030C ; [.21D3.0020.0002] # LATIN SMALL LETTER S WITH CARON
0160      ; [.21D3.0020.0008] # LATIN CAPITAL LETTER S WITH CARON
0053 030C ; [.21D3.0020.0008] # LATIN CAPITAL LETTER S WITH CARON
015B      ; [.21D4.0020.0002] # LATIN SMALL LETTER S WITH ACUTE
0073 0301 ; [.21D4.0020.0002] # LATIN SMALL LETTER S WITH ACUTE
0073 0341 ; [.21D4.0020.0002] # LATIN SMALL LETTER S WITH ACUTE
015A      ; [.21D4.0020.0008] # LATIN CAPITAL LETTER S WITH ACUTE
0053 0301 ; [.21D4.0020.0008] # LATIN CAPITAL LETTER S WITH ACUTE
0053 0341 ; [.21D4.0020.0008] # LATIN CAPITAL LETTER S WITH ACUTE
017E      ; [.2287.0020.0002] # LATIN SMALL LETTER Z WITH CARON
007A 030C ; [.2287.0020.0002] # LATIN SMALL LETTER Z WITH CARON
017D      ; [.2287.0020.0008] # LATIN CAPITAL LETTER Z WITH CARON
005A 030C ; [.2287.0020.0008] # LATIN CAPITAL LETTER Z WITH CARON
017A      ; [.2288.0020.0002] # LATIN SMALL LETTER Z WITH ACUTE
007A 0301 ; [.2288.0020.0002] # LATIN SMALL LETTER Z WITH ACUTE
007A 0341 ; [.2288.0020.0002] # LATIN SMALL LETTER Z WITH ACUTE
0179      ; [.2288.0020.0008] # LATIN CAPITAL LETTER Z WITH ACUTE
005A 0301 ; [.2288.0020.0008] # LATIN CAPITAL LETTER Z WITH ACUTE
005A 0341 ; [.2288.0020.0008] # LATIN CAPITAL LETTER Z WITH ACUTE
ENTRY
};
